@@ -1,20 +1,20 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2019-2022 A S Lewis
+# Copyright (C) 2019-2023 A S Lewis
 #
-# This program is free software: you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the Free Software
-# Foundation, either version 3 of the License, or (at your option) any later
-# version.
+# This program is free software; you can redistribute it and/or modify it under
+# the terms of the GNU Lesser General Public License as published by the Free
+# Software Foundation; either version 2.1 of the License, or (at your option)
+# any later version.
 #
 # This program is distributed in the hope that it will be useful, but WITHOUT
 # ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
+# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 # details.
 #
-# You should have received a copy of the GNU General Public License along with
-# this program. If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU Lesser General Public License
+# along with this program. If not, see <http://www.gnu.org/licenses/>.
 
 
 """Tidy operation classes."""
@@ -87,8 +87,8 @@ class TidyManager(threading.Thread):
                 not set should be removed from the database (no files are
                 deleted)
 
-            remove_dupe_flag: True if any media.Video objects, which are not
-                marked as downloaded and which share a URL with another
+            remove_duplicate_flag: True if any media.Video objects, which are
+                not marked as downloaded and which share a URL with another
                 media.Video object with the same parent and which is marked as
                 downloaded, should be removed from the database (no files are
                 deleted)
@@ -101,6 +101,8 @@ class TidyManager(threading.Thread):
 
             del_thumb_flag: True if all thumbnail files should be deleted
 
+            del_webp_flag: True if all .webp thumbnail files should be deleted
+
             convert_webp_flag: True if all .webp thumbnail files should be
                 converted to .jpg
 
@@ -113,6 +115,9 @@ class TidyManager(threading.Thread):
 
             del_xml_flag: True if all annotation files should be deleted
 
+            convert_ext_flag: True if .unknown_video file extensions should be
+                converted to .mp4 (experimental; see Git #472)
+
     """
 
 
@@ -168,13 +173,15 @@ class TidyManager(threading.Thread):
         #   and which share a URL with another media.Video object with the same
         #   parent and which is marked as downloaded, should be removed from
         #   the database (no files are deleted)
-        self.remove_dupe_flag = choices_dict['remove_dupe_flag']
+        self.remove_duplicate_flag = choices_dict['remove_duplicate_flag']
         # True if all youtube-dl archive files should be deleted
         self.del_archive_flag = choices_dict['del_archive_flag']
         # True if all thumbnail files should be moved into a subdirectory
         self.move_thumb_flag = choices_dict['move_thumb_flag']
         # True if all thumbnail files should be deleted
         self.del_thumb_flag = choices_dict['del_thumb_flag']
+        # True if all .webp thumbnail files should be deleted
+        self.del_webp_flag = choices_dict['del_webp_flag']
         # True if all .webp thumbnail files should be converted to .jpg.
         #   Requires mainapp.TartubeApp.ffmpeg_fail_flag set to False
         self.convert_webp_flag = choices_dict['convert_webp_flag']
@@ -187,6 +194,9 @@ class TidyManager(threading.Thread):
         self.del_json_flag = choices_dict['del_json_flag']
         # True if all annotation files should be deleted
         self.del_xml_flag = choices_dict['del_xml_flag']
+        # True if .unknown_video file extensions should be converted to .mp4
+        #   (experimental; see Git #472)
+        self.convert_ext_flag = choices_dict['convert_ext_flag']
 
         # The number of media data objects whose directories have been tidied
         #   so far...
@@ -203,15 +213,17 @@ class TidyManager(threading.Thread):
         self.video_deleted_count = 0
         self.other_deleted_count = 0
         self.remove_no_url_count = 0
-        self.remove_dupe_count = 0
+        self.remove_duplicate_count = 0
         self.archive_deleted_count = 0
         self.thumb_moved_count = 0
         self.thumb_deleted_count = 0
+        self.webp_deleted_count = 0
         self.webp_converted_count = 0
         self.data_moved_count = 0
         self.descrip_deleted_count = 0
         self.json_deleted_count = 0
         self.xml_deleted_count = 0
+        self.ext_converted_count = 0
 
 
         # Code
@@ -325,7 +337,7 @@ class TidyManager(threading.Thread):
             '   ' + _('Remove no_URL videos from database:') + ' ' + text,
         )
 
-        if self.remove_dupe_flag:
+        if self.remove_duplicate_flag:
             text = _('YES')
         else:
             text = _('NO')
@@ -366,6 +378,16 @@ class TidyManager(threading.Thread):
             '   ' + _('Delete all thumbnail files:') + ' ' + text,
         )
 
+        if self.del_webp_flag:
+            text = _('YES')
+        else:
+            text = _('NO')
+
+        self.app_obj.main_win_obj.output_tab_write_stdout(
+            1,
+            '   ' + _('Delete all .webp thumbnail files:') + ' ' + text,
+        )
+
         if self.convert_webp_flag:
             text = _('YES')
         else:
@@ -417,6 +439,17 @@ class TidyManager(threading.Thread):
             '   ' + _('Delete all annotation files:') + ' ' + text,
         )
 
+        if self.convert_ext_flag:
+            text = _('YES')
+        else:
+            text = _('NO')
+
+        self.app_obj.main_win_obj.output_tab_write_stdout(
+            1,
+            '   ' + _('Convert .unknown_video file extensions to .mp4:') \
+            + ' ' + text,
+        )
+
         # Compile a list of channels, playlists and folders to tidy up (each
         #   one has their own sub-directory inside Tartube's data directory)
         obj_list = []
@@ -424,9 +457,10 @@ class TidyManager(threading.Thread):
             # Add this channel/playlist/folder, and any child channels/
             #   playlists/folders (but not videos, obviously)
             obj_list = self.init_obj.compile_all_containers(obj_list)
+
         else:
             # Add all channels/playlists/folders in the database
-            for dbid in list(self.app_obj.media_name_dict.values()):
+            for dbid in self.app_obj.container_reg_dict.keys():
 
                 obj = self.app_obj.media_reg_dict[dbid]
                 # Don't add private folders
@@ -503,13 +537,13 @@ class TidyManager(threading.Thread):
                 + str(self.remove_no_url_count),
             )
 
-        if self.remove_dupe_flag:
+        if self.remove_duplicate_flag:
 
             self.app_obj.main_win_obj.output_tab_write_stdout(
                 1,
                 '   ' \
                 + _('Undownloaded duplicate videos removed from database:') \
-                + ' ' + str(self.remove_dupe_count),
+                + ' ' + str(self.remove_duplicate_count),
             )
 
         if self.del_archive_flag:
@@ -536,6 +570,14 @@ class TidyManager(threading.Thread):
                 + str(self.thumb_deleted_count),
             )
 
+        if self.del_webp_flag:
+
+            self.app_obj.main_win_obj.output_tab_write_stdout(
+                1,
+                '   ' + _('.webp thumbnail files deleted:') + ' ' \
+                + str(self.webp_deleted_count),
+            )
+
         if self.convert_webp_flag:
 
             self.app_obj.main_win_obj.output_tab_write_stdout(
@@ -576,6 +618,14 @@ class TidyManager(threading.Thread):
                 + str(self.xml_deleted_count),
             )
 
+        if self.convert_ext_flag:
+
+            self.app_obj.main_win_obj.output_tab_write_stdout(
+                1,
+                '   ' + _('File extensions converted:') + ' ' \
+                + str(self.ext_converted_count),
+            )
+
         # Let the timer run for a few more seconds to prevent Gtk errors
         GObject.timeout_add(
             0,
@@ -613,6 +663,9 @@ class TidyManager(threading.Thread):
             _('Checking:') + ' \'' + media_data_obj.name + '\'',
         )
 
+        if self.convert_ext_flag:
+            self.convert_file_ext(media_data_obj)
+
         if self.corrupt_flag:
             self.check_video_corrupt(media_data_obj)
 
@@ -625,8 +678,8 @@ class TidyManager(threading.Thread):
         if self.remove_no_url_flag:
             self.remove_no_url(media_data_obj)
 
-        if self.remove_dupe_flag:
-            self.remove_dupe(media_data_obj)
+        if self.remove_duplicate_flag:
+            self.remove_duplicate(media_data_obj)
 
         if self.del_archive_flag:
             self.delete_archive(media_data_obj)
@@ -637,6 +690,9 @@ class TidyManager(threading.Thread):
         if self.del_thumb_flag:
             self.delete_thumb(media_data_obj)
 
+        if self.del_webp_flag:
+            self.delete_webp(media_data_obj)
+
         if self.convert_webp_flag:
             self.convert_webp(media_data_obj)
 
@@ -653,6 +709,62 @@ class TidyManager(threading.Thread):
             self.delete_xml(media_data_obj)
 
 
+    def convert_file_ext(self, media_data_obj):
+
+        """Called by self.tidy_directory().
+
+        Git #472: yt-dlp occasionally downloads .mp4 videos from VK (and
+        possibly other websites) whose file extension is mistakenly set to
+        .mp4. Convert any such files to .mp4.
+
+        This is an experimental/temporary feature.
+
+        Args:
+
+            media_data_obj (media.Channel, media.Playlist or media.Folder):
+                The media data object whose directory must be tidied up
+
+        """
+
+        # Import the main window (for convenience)
+        main_win_obj = self.app_obj.main_win_obj
+
+        # Get a list of video files in the directory
+        container_path = media_data_obj.get_actual_dir(self.app_obj)
+        try:
+            init_list = os.listdir(container_path)
+        except:
+            # Can't read the directory
+            return
+
+        # Find all .unknown_video files
+        for relative_path in init_list:
+
+            # (If self.stop_tidy_operation() has been called, give up
+            #   immediately)
+            if not self.running_flag:
+                return
+
+            file_path = os.path.abspath(
+                os.path.join(container_path, relative_path),
+            )
+
+            if os.path.isfile(file_path):
+
+                filename, ext = os.path.splitext(relative_path)
+                if ext == '.unknown_video':
+                    mod_file_path = re.sub(
+                        r'\.unknown_video$',
+                        '.mp4',
+                        file_path,
+                    )
+                    self.app_obj.move_file_or_directory(
+                        file_path,
+                        mod_file_path,
+                    )
+                    self.ext_converted_count += 1
+
+
     def check_video_corrupt(self, media_data_obj):
 
         """Called by self.tidy_directory().
@@ -917,7 +1029,7 @@ class TidyManager(threading.Thread):
                 self.remove_no_url_count += 1
 
 
-    def remove_dupe(self, media_data_obj):
+    def remove_duplicate(self, media_data_obj):
 
         """Called by self.tidy_directory().
 
@@ -962,7 +1074,7 @@ class TidyManager(threading.Thread):
                         duplicate_obj,
                     )
 
-                    self.remove_dupe_count += 1
+                    self.remove_duplicate_count += 1
 
 
     def delete_archive(self, media_data_obj):
@@ -1095,6 +1207,50 @@ class TidyManager(threading.Thread):
                     self.thumb_deleted_count += 1
 
 
+    def delete_webp(self, media_data_obj):
+
+        """Called by self.tidy_directory().
+
+        Checks all child videos of the specified media data object. If the
+        associated .webp thumbnail file exists, delete it.
+
+        Args:
+
+            media_data_obj (media.Channel, media.Playlist or media.Folder):
+                The media data object whose directory must be tidied up
+
+        """
+
+        for video_obj in media_data_obj.compile_all_videos( [] ):
+
+            if video_obj.file_name is not None:
+
+                # Thumbnails might be in one of two locations
+                webp_path = utils.find_thumbnail_webp_strict(
+                    self.app_obj,
+                    video_obj
+                )
+
+                # If the video's parent container has an alternative download
+                #   destination set, we must check the corresponding media
+                #   data object. If the latter also has a media.Video object
+                #   matching this video, then this function returns None and
+                #   nothing is deleted
+                if webp_path is not None:
+
+                    webp_path = self.check_video_in_actual_dir(
+                        media_data_obj,
+                        video_obj,
+                        webp_path,
+                    )
+
+                # Delete the thumbnail file
+                if webp_path is not None \
+                and os.path.isfile(webp_path) \
+                and self.app_obj.remove_file(webp_path):
+                    self.webp_deleted_count += 1
+
+
     def convert_webp(self, media_data_obj):
 
         """Called by self.tidy_directory().
@@ -1115,7 +1271,10 @@ class TidyManager(threading.Thread):
             if video_obj.file_name is not None:
 
                 # Thumbnails might be in one of four locations
-                thumb_path = utils.find_thumbnail_webp(self.app_obj, video_obj)
+                thumb_path = utils.find_thumbnail_webp_intact_or_broken(
+                    self.app_obj,
+                    video_obj,
+                )
 
                 # If the video's parent container has an alternative download
                 #   destination set, we must check the corresponding media
@@ -1433,7 +1592,7 @@ class TidyManager(threading.Thread):
             delete_path (str): The path to a file which the calling function
                 wants to delete
 
-        Returns:
+        Return values:
 
             The specified delete_path if it can be deleted, or None if it
                 should not be deleted
